//=============================================================================//
//  ___  ___   _   _   _    __   _   ___ ___ __ __
// |_ _|| __| / \ | \_/ |  / _| / \ | o \ o \\ V /
//  | | | _| | o || \_/ | ( |_n| o ||   /   / \ / 
//  |_| |___||_n_||_| |_|  \__/|_n_||_|\\_|\\ |_|  2006
//										 
//=============================================================================//

#include <windows.h>
#include "common/GMLuaModule.h"
#include "mIRC_Interface.h"

// This is a macro to make life easier.
// When the dll is loaded it will call the function (Init)
GMOD_MODULE( Init, Shutdown );


//
// A simple messagebox example
//

// What we do to startup everything (find the mIRC HWND and setup the mapped file)
LUA_FUNCTION( lSetup )
{
	// Simple!
	girc.Init();

	return 0;
}


// Once we are done we should shut down (good practice to do!) clean up crap
LUA_FUNCTION( lShutDown )
{
	// Simple!
	girc.ShutDown();

	return 0;
}

// Is the mIRC process open and the handle valid? If you call a request to it without it
// being open you will blow the hell up!
// { return bool }
LUA_FUNCTION( lIsmIRCOpen )
{
	// Oh kay
	bool bRet = girc.IsmIRCOpen();

	g_Lua->Push(bRet);

	return 1;
}


// A raw evaluation function, so you can query mIRC with anything you want that I did not include
// [ string in ] { return string }
LUA_FUNCTION( lRawEval )
{
	const char* pszQuery = g_Lua->GetString(1);

	char szBuffer[MAX_BUFFER_SIZE];

	girc.RawEval(const_cast<char*>(pszQuery), szBuffer);

	g_Lua->Push(szBuffer);

	return 1;
}


// Same thing as RawEval, only it is just a raw send, 
// (as if it was typed into your chat box in mIRC)
// [ string in ]
LUA_FUNCTION( lRawSend )
{
	const char* pszQuery = g_Lua->GetString(1);
	girc.RawSend(const_cast<char*>(pszQuery));

	return 0;
}


// This simply tells us how many networks we have open
// { return int totalnets }
LUA_FUNCTION( lGetTotalNets )
{
	int iRet = girc.GetTotalNets();

	g_Lua->Push((float)iRet);

	return 1;
}

// What is the name of the net we are currently on? 
// [ int netid ] { return string netname }
LUA_FUNCTION( lGetNetName )
{
	char szBuffer[MAX_BUFFER_SIZE];
	int iNet = g_Lua->GetInteger(1);

	girc.GetNetName(iNet, szBuffer);

	g_Lua->Push(szBuffer);

	return 1;

}


// Returns the ID of our active net
// { return int netid }
LUA_FUNCTION( lGetActiveNet )
{

	int iRet = girc.GetActiveNet();

	g_Lua->Push((float)iRet);

	return 1;

}

// How many channels do you have open inside a given network?
// [ int netid ] { return int totalchans }
LUA_FUNCTION( lGetTotalChannels )
{
	int iNet = g_Lua->GetInteger(1);

	int iRet = girc.GetTotalChannels(iNet);

	g_Lua->Push((float)iRet);

	return 1;
}


LUA_FUNCTION( lGetChannelName )
{

	char szBuffer[MAX_BUFFER_SIZE];

	int iNet = g_Lua->GetInteger(1);
	int iChan = g_Lua->GetInteger(2);

	girc.GetChannelName(iNet, iChan, szBuffer);

	g_Lua->Push(szBuffer);

	return 1;

}

// Returns the ID of our currently active channel
// { return int activechanid }
LUA_FUNCTION( lGetActiveChannel )
{

	int iRet = girc.GetActiveChannel();

	g_Lua->Push((float)iRet);

	return 1;

}

// Sets your currently active channel
// [ int netid, int chanid ]
LUA_FUNCTION( lSetActiveChannel )
{

	int iNet = g_Lua->GetInteger(1);
	int iChan = g_Lua->GetInteger(2);

	girc.SetActiveChannel(iNet, iChan);

	return 0;

}

// Sends chat/text to your currently active channel
// [ string in ]
LUA_FUNCTION( lSendToActiveChannel )
{

	const char* pszBuf = g_Lua->GetString(1);

	girc.SendToActiveChannel(const_cast<char*>(pszBuf));

	return 0;

}


// Send text to a channel of your choosing
// [ int netid, int chanid, string texttosend ]
LUA_FUNCTION( lSendTextToChannel )
{
	int iNet = g_Lua->GetInteger(1);
	int iChan = g_Lua->GetInteger(2);
	const char* pszBuffer = g_Lua->GetString(3);

	girc.SendTextToChannel(iNet, iChan, const_cast<char*>(pszBuffer));
    
	return 0;

}

// Tells you how many users on a given channel on a given network :P
// [ int netid, int chanid ] { return int usercount }
LUA_FUNCTION( lGetTotalUsers )
{

	int iNet = g_Lua->GetInteger(1);
	int iChan = g_Lua->GetInteger(2);

	int iRet = girc.GetTotalUsers(iNet, iChan);

	g_Lua->Push((float)iRet);

	return 1;

}

// Gets the username from a channel
// [ int netid, int chanid, int userid ] { return string username }
LUA_FUNCTION( lGetUserName )
{
	int iNet = g_Lua->GetInteger(1);
	int iChan = g_Lua->GetInteger(2);
	int iUser = g_Lua->GetInteger(3);

	char szBuffer[MAX_BUFFER_SIZE];

	girc.GetUserName(iNet, iChan, iUser, szBuffer);

	g_Lua->Push(szBuffer);

	return 1;

}


// How many lines do we have in a channel?
// [ int netid, int chanid ] { return int linecount }
LUA_FUNCTION( lGetTotalLines )
{
	int iNet = g_Lua->GetInteger(1);
	int iChan = g_Lua->GetInteger(2);

	int iRet = girc.GetTotalLines(iNet, iChan);

	g_Lua->Push((float)iRet);

	return 1;
}


LUA_FUNCTION( lGetLine )
{

	int iNet = g_Lua->GetInteger(1);
	int iChan = g_Lua->GetInteger(2);
	int iLine = g_Lua->GetInteger(3);

	char szBuffer[MAX_BUFFER_SIZE];

	girc.GetLine(iNet, iChan, iLine, szBuffer);

	g_Lua->Push(szBuffer);

	return 1;


}


// When the DLL is initialized it will call this function to set itself up.
// It should be used to set up your library so it's all ready to go.
int Init( void )
{
	// Lets make a global table filled with functions (a module)
	g_Lua->NewGlobalTable( "gIRC" );
	ILuaObject* table = g_Lua->GetGlobal( "gIRC" );

		// In case mIRC was not open when we started
		table->SetMember( "SetupInterface", lSetup );
		table->SetMember( "ShutDownInterface", lShutDown );

		// Make the Lua user responsible for checking if mIRC is running, 
		// I don't really want to check every single function, 
		// this crap has enough overhead as is!
		table->SetMember( "IsMircOpen", lIsmIRCOpen );


		// Raw evaluation/send functions
		table->SetMember( "RawEval", lRawEval );
		table->SetMember( "RawSend", lRawSend );

		// Net/Server management
		table->SetMember( "GetTotalNets", lGetTotalNets );
		table->SetMember( "GetNetName", lGetNetName );
		table->SetMember( "GetActiveNet", lGetActiveNet );


		// Channel management
		table->SetMember( "GetTotalChannels", lGetTotalChannels );
		table->SetMember( "GetChannelName", lGetChannelName );
		table->SetMember( "GetActiveChannel", lGetActiveChannel );
		table->SetMember( "SetActiveChannel", lSetActiveChannel );
		table->SetMember( "SendToActiveChannel", lSendToActiveChannel );
		table->SetMember( "SendTextToChannel", lSendTextToChannel );

		// User management
		table->SetMember( "GetTotalUsers", lGetTotalUsers );
		table->SetMember( "GetUserName", lGetUserName );

		// Lines from channels
		table->SetMember( "GetTotalLines", lGetTotalLines );
		table->SetMember( "GetLine", lGetLine );

	// We're not deleting the table here - we're deleting the reference to it.
	// If you don't do this you're leaking memory.
	SAFE_UNREF( table );

	return 0;
}

// When the DLL is shutdown it calls this function.
// g_Lua should be assumed to have been released here
int Shutdown( void )
{
	girc.ShutDown();
	return 0;
}

